#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from z3c.language.session.interfaces import ILanguageSession

# import local interfaces

# import Zope3 packages

# import local packages
from z3c.jsonrpc.publisher import MethodPublisher


class I18nMethodsPublisher(MethodPublisher):
    """I18n methods publisher"""

    def setUserLanguage(self, lang):
        if lang:
            ILanguageSession(self.request).setLanguage(lang)



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from z3c.form.interfaces import HIDDEN_MODE, IErrorViewSnippet
from z3c.json.interfaces import IJSONWriter
from zope.authentication.interfaces import IAuthentication
from zope.component.interfaces import ISite
from zope.publisher.browser import BrowserPage
from zope.security.interfaces import IUnauthorized
from zope.session.interfaces import ISession

# import local interfaces
from ztfy.baseskin.interfaces import IDefaultView, IDialog
from ztfy.myams.interfaces import IModalFullPage

# import Zope3 packages
from z3c.form import field, button
from z3c.formjs import ajax
from zope.component import getUtility, getUtilitiesFor, getMultiAdapter, queryMultiAdapter
from zope.interface import implements, Interface, Invalid
from zope.schema import TextLine, Password
from zope.site import hooks
from zope.traversing.browser.absoluteurl import absoluteURL

# import local packages
from ztfy.myams.form import AddForm
from ztfy.utils.traversing import getParent

from ztfy.myams import _


class ILoginFormFields(Interface):
    """Login form fields interface"""

    username = TextLine(title=_("login-field", "Login"),
                        description=_("Principal ID"),
                        required=True)

    password = Password(title=_("password-field", "Password"),
                        required=True)

    came_from = TextLine(title=_("camefrom-field", "Login origin"),
                         required=False)


class ILoginFormButtons(Interface):
    """Login form buttons interface"""

    login = button.Button(name='login', title=_("Login"))


class LoginView(AddForm):
    """Login view"""

    implements(IModalFullPage)

    legend = _("Please enter valid credentials to login")
    dialog_class = 'modal-medium'

    fields = field.Fields(ILoginFormFields)
    buttons = button.Buttons(ILoginFormButtons)
    handler = '/@@ajax/handleLogin'
    principal = None

    def updateWidgets(self, prefix=None):
        super(LoginView, self).updateWidgets(prefix)
        self.widgets['came_from'].mode = HIDDEN_MODE
        origin = self.request.get('came_from') or self.request.get(self.prefix + self.widgets.prefix + 'came_from')
        if not origin:
            origin = self.request.getURL()
            stack = self.request.getTraversalStack()
            if stack:
                origin += '/' + '/'.join(stack[::-1])
        self.widgets['came_from'].value = origin

    def updateActions(self):
        super(AddForm, self).updateActions()
        if 'login' in self.actions:
            self.actions['login'].addClass('btn-primary')

    def extractData(self, setErrors=True):
        data, errors = super(LoginView, self).extractData(setErrors)
        if errors:
            self.logout()
            return data, errors
        self.request.form['login'] = data['username']
        self.request.form['password'] = data['password']
        context = getParent(self.context, ISite)
        while context is not None:
            old_site = hooks.getSite()
            try:
                hooks.setSite(context)
                for name, auth in getUtilitiesFor(IAuthentication):
                    try:
                        self.principal = auth.authenticate(self.request)
                        if self.principal is not None:
                            return data, errors
                    except:
                        continue
            finally:
                hooks.setSite(old_site)
            context = getParent(context, ISite, allow_context=False)
        if self.principal is None:
            error = Invalid(_("Invalid credentials"))
            view = getMultiAdapter((error, self.request, None, None, self, self.context),
                                   IErrorViewSnippet)
            view.update()
            errors += (view, )
            if setErrors:
                self.widgets.errors = errors
                self.logout()
        return data, errors

    @ajax.handler
    def handleLogin(self):
        writer = getUtility(IJSONWriter)
        self.updateWidgets()
        data, errors = self.extractData()
        if errors:
            return writer.write(self.getAjaxErrors())
        if self.principal is not None:
            if IUnauthorized.providedBy(self.context):
                context, _layer, _permission = self.context.args
                return writer.write({'status': 'redirect',
                                     'location': absoluteURL(context, self.request)})
            else:
                came_from = data.get('came_from')
                if came_from:
                    return writer.write({'status': 'redirect',
                                         'location': came_from})
                else:
                    target = queryMultiAdapter((self.context, self.request, Interface), IDefaultView)
                    return writer.write({'status': 'redirect',
                                         'location': '%s/%s' % (absoluteURL(self.context, self.request),
                                                                target.viewname if target is not None else '@@index.html')})
        else:
            return writer.write({'status': 'redirect',
                                 'location': '%s/@@login.html?came_from=%s' % (absoluteURL(self.context, self.request),
                                                                               data.get('came_from'))})

    def logout(self):
        try:
            sessionData = ISession(self.request)['zope.pluggableauth.browserplugins']
            sessionData['credentials'] = None
        except KeyError:
            pass
        self.principal = None


class LoginDialogView(LoginView):
    """Login dialog view"""

    implements(IDialog)


class LogoutView(BrowserPage):
    """Logout view"""

    def __call__(self):
        context = getParent(self.context, ISite)
        while context is not None:
            old_site = hooks.getSite()
            try:
                hooks.setSite(context)
                for _name, auth in getUtilitiesFor(IAuthentication):
                    auth.logout(self.request)
            finally:
                hooks.setSite(old_site)
            context = getParent(context, ISite, allow_context=False)
        target = queryMultiAdapter((self.context, self.request, Interface), IDefaultView)
        self.request.response.redirect('%s/%s' % (absoluteURL(self.context, self.request),
                                                  target.viewname if target is not None else '@@index.html'))
        return u''



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from z3c.json.interfaces import IJSONWriter
from zope.tales.interfaces import ITALESFunctionNamespace

# import local interfaces
from ztfy.myams.interfaces import IMyAMSApplication, IObjectData
from ztfy.myams.interfaces.configuration import IMyAMSConfiguration
from ztfy.myams.tal.interfaces import IMyAMSTalesAPI

# import Zope3 packages
from zope.component import getUtility
from zope.interface import implements
from zope.security.proxy import removeSecurityProxy

# import local packages
from ztfy.utils.traversing import getParent


class MyAMSTalesAdapter(object):
    """myams: TALES adapter"""

    implements(IMyAMSTalesAPI, ITALESFunctionNamespace)

    def __init__(self, context):
        self.context = context

    def setEngine(self, engine):
        self.request = engine.vars['request']

    def data(self):
        data = IObjectData(self.context, None)
        if (data is not None) and data.object_data:
            writer = getUtility(IJSONWriter)
            return writer.write(data.object_data)

    def configuration(self):
        application = getParent(self.context, IMyAMSApplication)
        if application is not None:
            return IMyAMSConfiguration(application, None)

    def resources(self):
        application = getParent(self.context, IMyAMSApplication)
        if application is not None:
            for resource in application.resources:
                removeSecurityProxy(resource).need()



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from zope.viewlet.interfaces import IViewlet, IViewletManager

# import local interfaces

# import Zope3 packages
from zope.interface import Attribute
from zope.schema import TextLine, Bool

# import local packages

from ztfy.myams import _


class IShortcutsViewletManager(IViewletManager):
    """Shortcuts viewlet manager interface"""


class IShortcutViewlet(IViewlet):
    """Shortcut viewlet interface"""

    bg_color_class = TextLine(title=_("Shortcut CSS class"))

    icon_class = TextLine(title=_("Shortcut icon class"))

    label = Attribute(_("Shortcut title"))

    href = TextLine(title=_("Shortcut target URL"))

    modal_target = Bool(title=_("Modal target"),
                        required=True,
                        default=True)

    checked = TextLine(title=_("Checked shortcut class"),
                       required=False)

    def getURL(self):
        """Return shortcut absolute URL"""



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.myams.interfaces.configuration import IMyAMSConfiguration
from ztfy.skin.interfaces import IPropertiesMenuTarget

# import Zope3 packages
from z3c.form import field
from zope.interface import implements
from zope.traversing.namespace import view

# import local packages
from ztfy.skin.form import EditForm
from ztfy.skin.menu import MenuItem

from ztfy.myams import _


class MyAMSConfigurationNamespaceTraverser(view):
    """++configuration++ namespace"""

    def traverse(self, name, ignored):
        return IMyAMSConfiguration(self.context)


class MyAMSConfigurationEditForm(EditForm):
    """MyAMS main configuration edit form"""

    implements(IPropertiesMenuTarget)

    legend = _("Application properties")
    fields = field.Fields(IMyAMSConfiguration)

    def getContent(self):
        return IMyAMSConfiguration(self.context)


class MyAMSConfigurationMenuItem(MenuItem):
    """MyAMS configuration menu item"""

    title = _("Properties")



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from z3c.language.negotiator.interfaces import INegotiatorManager
from zope.i18n.interfaces import INegotiator

# import local interfaces

# import Zope3 packages
from zope.component import adapts, queryUtility
from zope.i18n import translate
from zope.interface import Interface

# import local packages
from ztfy.baseskin.viewlet import ContentProviderBase
from ztfy.myams.layer import MyAMSLayer


class FlagsContentProvider(ContentProviderBase):
    """Flags content provider"""

    adapts(Interface, MyAMSLayer, Interface)

    @property
    def langs(self):
        manager = queryUtility(INegotiatorManager)
        if manager is not None:
            return manager.offeredLanguages
        return ()

    def getLabel(self, lang):
        try:
            from ztfy.i18n.languages import BASE_LANGUAGES
        except ImportError:
            return lang
        else:
            return translate(BASE_LANGUAGES.get(lang), context=self.request)

    @property
    def current(self):
        manager = queryUtility(INegotiatorManager)
        negotiator = queryUtility(INegotiator)
        if (manager is not None) and (negotiator is not None):
            return negotiator.getLanguage(manager.offeredLanguages, self.request)
        return None



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from zope.viewlet.interfaces import IViewlet, IViewletManager

# import local interfaces

# import Zope3 packages
from zope.schema import TextLine, List, Object, Dict

# import local packages

from ztfy.myams import _


class ITopLinksViewletManager(IViewletManager):
    """Top links viewlet manager interface"""


class ITopLinksMenu(IViewlet):
    """Top link menu"""

    css_class = TextLine(title=_("Menu CSS class"),
                         required=False)

    label = TextLine(title=_("Menu label"))

    click_handler = TextLine(title=_("Menu click handler"))

    url = TextLine(title=_("Menu link location"))

    data = Dict(title=_("Menu data attributes"),
                key_type=TextLine(),
                value_type=TextLine())


class ITopLinksViewlet(IViewlet):
    """Top link viewlet"""

    label = TextLine(title=_("Main label"))

    dropdown_label = TextLine(title=_("Dropdown menu label"))

    css_class = TextLine(title=_("Main CSS class"))

    viewlets = List(title=_("Top links menus"),
                    value_type=Object(schema=ITopLinksMenu))



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from zope.interface import Interface

# import local packages


class IMyAMSTalesAPI(Interface):
    """myams: TALES namespace interface"""

    def data(self):
        """Get context data"""

    def configuration(self):
        """Get application configuration"""

    def resources(self):
        """Include application's Fanstatic resources"""



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from zope.viewlet.interfaces import IViewletManager, IViewlet

# import local interfaces

# import Zope3 packages
from zope.schema import TextLine, Bool, Dict

# import local packages

from ztfy.myams import _


class IMainMenusViewletManager(IViewletManager):
    """Main menu viewlet manager"""


class IMenu(IViewlet):
    """Menu interface"""

    header = TextLine(title=_("Menu header"),
                      required=False)


class IMenuItem(IViewlet):
    """Menu item interface

    A menu is a viewlet as well as a viewlets manager,
    containing sub-menus
    """

    css_class = TextLine(title=_("CSS class"),
                         required=False)

    icon_class = TextLine(title=_("Icon CSS class"),
                          required=False)

    label = TextLine(title=_("Menu title"),
                     required=False)

    badge = TextLine(title=_("Badge text"),
                     required=False)

    badge_class = TextLine(title=_("Badge CSS class"),
                           required=False)

    notice = TextLine(title=_("Notice text"),
                      required=False)

    notice_class = TextLine(title=_("Notice CSS class"),
                            required=False)

    click_handler = TextLine(title=_("Menu click handler"),
                             required=False)

    url = TextLine(title=_("Menu link location"),
                   required=False,
                   default=u'#')

    modal_target = Bool(title=_("Modal target"),
                        required=False,
                        default=False)

    data = Dict(title=_("Menu data attributes"),
                key_type=TextLine(),
                value_type=TextLine(),
                required=False)

    def getURL(self):
        """Get menu target URL"""

    def getDataAttributes(self):
        """Get data attributes"""



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from zope.interface import Interface, Attribute
from zope.schema import TextLine, Bool

# import local packages
from ztfy.file.schema import ImageField, FileField

from ztfy.myams import _


class IMyAMSStaticConfiguration(Interface):
    """MyAMS static configuration"""

    application_package = TextLine(title=_("Main application package"),
                                   description=_("This package is used to get application version"),
                                   required=False)

    application_name = TextLine(title=_("Main application name"),
                                description=_("This name is used to display application version"),
                                required=False)

    version = Attribute(_("Application version"))

    include_top_links = Bool(title=_("Include top links?"),
                             default=True,
                             required=True)

    include_site_search = Bool(title=_("Include site search?"),
                               default=True,
                               required=True)

    include_mobile_search = Bool(title=_("Include mobile search?"),
                                 default=True,
                                 required=True)

    include_user_activity = Bool(title=_("Include user dropdown window?"),
                                 default=True,
                                 required=True)

    include_user_shortcuts = Bool(title=_("Include user shortcuts?"),
                                  default=True,
                                  required=True)

    include_logout_button = Bool(title=_("Include logout button?"),
                                 default=True,
                                 required=True)

    include_minify_button = Bool(title=_("Include minify button?"),
                                 default=True,
                                 required=True)

    include_flags = Bool(title=_("Include flags menu?"),
                         default=True,
                         required=True)

    include_menus = Bool(title=_("Include main menus?"),
                         default=True,
                         required=True)

    include_ribbon = Bool(title=_("Include ribbon?"),
                          default=True,
                          required=True)

    include_reload_button = Bool(title=_("Include reload button?"),
                                 default=True,
                                 required=True)

    body_css_class = TextLine(title=_("Body HTML tag CSS class"),
                              required=False)


class IMyAMSConfiguration(Interface):
    """MyAMS application global configuration"""

    title = TextLine(title=_("Title"),
                     description=_("Application title displayed in title bar"),
                     required=False)

    description = TextLine(title=_("Description"),
                           description=_("Main application description"),
                           required=False)

    author = TextLine(title=_("Author"),
                      description=_("Public author name"),
                      required=False)

    icon = ImageField(title=_("Favorites icon"),
                      description=_("Please provide a transparent image of 32x32 pixels..."),
                      required=False)

    logo = ImageField(title=_("Logo"),
                      description=_("Please provide a transparent image in PNG format..."),
                      required=False)

    logo_title = TextLine(title=_("Logo title"),
                          description=_("This text will be used as logo alternate text"),
                          required=False)

    custom_css = FileField(title=_("Custom CSS file"),
                           required=False)

    custom_js = FileField(title=_("Custom javascript file"),
                          required=False)

    google_analytics_key = TextLine(title=_("Google Analytics key"),
                                    required=False)

    static_configuration = Attribute(_("Application static configuration utility"))


class IMyAMSConfigurationTarget(Interface):
    """MyAMS configuration marker interface"""



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2013 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages
from fanstatic import Library, Resource

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages

# import local packages


from zope.i18nmessageid import MessageFactory
_ = MessageFactory('ztfy.myams')


library = Library('ztfy.myams', 'resources')

bootstrap_css = Resource(library, 'css/ext/bootstrap-3.1.1.css',
                         minified='css/ext/bootstrap-3.1.1.min.css')
bootstrap_modal_css = Resource(library, 'css/ext/bootstrap-modal.css',
                               minified='css/ext/bootstrap-modal.min.css',
                               depends=[bootstrap_css, ])

awesomefonts_css = Resource(library, 'css/ext/font-awesome-4.0.3.css',
                            minified='css/ext/font-awesome-4.0.3.min.css',
                            depends=[bootstrap_css, ])

myams_css = Resource(library, 'css/myams.css',
                     minified='css/myams.min.css',
                     depends=[bootstrap_modal_css, awesomefonts_css])

jquery = Resource(library, 'js/ext/jquery-2.1.0.js',
                  minified='js/ext/jquery-2.1.0.min.js',
                  bottom=True)

jquery_ui = Resource(library, 'js/ext/jquery-ui-1.10.3.min.js',
                     depends=[jquery, ],
                     bottom=True)

jquery_dataTables = Resource(library, 'js/ext/jquery-dataTables-1.9.4.js',
                             minified='js/ext/jquery-dataTables-1.9.4.min.js',
                             depends=[jquery, ],
                             bottom=True)

jquery_dataTables_tableTools = Resource(library, 'js/ext/jquery-dataTables-tableTools.js',
                                        minified='js/ext/jquery-dataTables-tableTools.min.js',
                                        depends=[jquery_dataTables, ],
                                        bottom=True)

bootstrap = Resource(library, 'js/ext/bootstrap-3.1.1.js',
                     minified='js/ext/bootstrap-3.1.1.min.js',
                     depends=[jquery, jquery_ui, bootstrap_css, bootstrap_modal_css],
                     bottom=True)

myams = Resource(library, 'js/myams.js',
                 minified='js/myams.min.js',
                 depends=[bootstrap, myams_css],
                 bottom=True)



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.myams.viewlet.shortcuts.interfaces import IShortcutsViewletManager, IShortcutViewlet

# import Zope3 packages
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty

# import local packages
from ztfy.baseskin.viewlet import WeightViewletManagerBase, ViewletBase

from ztfy.myams import _


class ShortcutsViewletManager(WeightViewletManagerBase):
    """Shortcuts viewlet manager"""

    implements(IShortcutsViewletManager)


class Shortcut(ViewletBase):
    """Shortcut viewlet"""

    implements(IShortcutViewlet)

    bg_color_class = FieldProperty(IShortcutViewlet['bg_color_class'])
    icon_class = FieldProperty(IShortcutViewlet['icon_class'])
    href = FieldProperty(IShortcutViewlet['href'])
    modal_target = FieldProperty(IShortcutViewlet['modal_target'])
    checked = FieldProperty(IShortcutViewlet['checked'])

    def getURL(self):
        return self.href


class ManagementInterfaceShortcut(Shortcut):
    """Access to management interface"""

    bg_color_class = 'bg-color-orangeDark'
    icon_class = 'fa-gears'
    label = _("Management interface")
    href = '@@properties.html'
    modal_target = False
    checked = False



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from zope.interface import Interface
from zope.schema import TextLine

# import local packages

from ztfy.myams import _


class IMyAMSApplicationSearch(Interface):
    """MyAMS application search configuration interface"""

    site_search_placeholder = TextLine(title=_("Site search placeholder"),
                                       required=False,
                                       default=_("Search..."))

    site_search_handler = TextLine(title=_("Site search handler"),
                                   required=False,
                                   default=u"#search.html")

    mobile_search_placeholder = TextLine(title=_("Mobile search placeholder"),
                                         required=False,
                                         default=_("Search..."))

    mobile_search_handler = TextLine(title=_("Mobile search handler"),
                                     required=False,
                                     default=u'#search.html')



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from ztfy.baseskin.interfaces import ISkinnable
from ztfy.baseskin.interfaces.form import checkSubmitButton

# import local interfaces

# import Zope3 packages
from z3c.form import button
from zope.interface import Interface, Attribute
from zope.schema import TextLine, Dict

# import local packages
from ztfy.baseskin.schema import CloseButton, ResetButton

from ztfy.myams import _


class IMyAMSApplication(ISkinnable):
    """MyAMS application interface"""

    configuration_name = TextLine(title=_("Application static configuration name"),
                                  required=True)

    resources = Attribute(_("Fanstatic static resources"))


class IObjectData(Interface):
    """Object custom data interface"""

    object_data = Dict(title=_("Data associated with this object"),
                       required=False)


class IFullPage(Interface):
    """Full page marker interface"""


class IModalFullPage(IFullPage):
    """Full page modal dialog marker interface"""

    dialog_class = Attribute(_("Default dialog CSS class"))


class IInnerPage(Interface):
    """Inner page marker interface"""


class IModalPage(Interface):
    """Modal page marker interface"""


class IAddFormButtons(Interface):
    """Default add form buttons"""

    reset = ResetButton(name='reset', title=_("Reset"))
    add = button.Button(name='add', title=_("Add"), condition=checkSubmitButton)


class IModalAddFormButtons(Interface):
    """Modal add form buttons"""

    close = CloseButton(name='close', title=_("Close"))
    add = button.Button(name='add', title=_("Add"), condition=checkSubmitButton)


class IEditFormButtons(Interface):
    """Default edit form buttons"""

    reset = ResetButton(name='reset', title=_("Reset"))
    submit = button.Button(name='submit', title=_("Submit"), condition=checkSubmitButton)


class IModalEditFormButtons(Interface):
    """Modal edit form buttons"""

    close = CloseButton(name='close', title=_("Close"))
    submit = button.Button(name='submit', title=_("Submit"), condition=checkSubmitButton)


class IModalDisplayFormButtons(Interface):
    """Modal display form buttons"""

    close = CloseButton(name='close', title=_("Close"))



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages
from datetime import datetime

# import Zope3 interfaces
from persistent.interfaces import IPersistent
from z3c.json.interfaces import IJSONWriter
from z3c.template.interfaces import ILayoutTemplate
from zope.authentication.interfaces import IAuthentication
from zope.pagetemplate.interfaces import IPageTemplate

# import local interfaces
from ztfy.myams.interfaces import IInnerPage, IModalPage

# import Zope3 packages
from z3c.template.template import getPageTemplate, getLayoutTemplate
from zope.component import getMultiAdapter, getUtility
from zope.i18n import translate
from zope.interface import implements
from zope.publisher.browser import BrowserPage
from zope.traversing.browser.absoluteurl import absoluteURL

# import local packages
from ztfy.utils.date import formatDatetime
from ztfy.utils.text import textToHTML
from ztfy.utils.timezone import tztime
from ztfy.utils.traversing import getParent

from ztfy.myams import _


class BaseTemplateBasedPage(BrowserPage):
    """Base template based page"""

    template = getPageTemplate()

    def __call__(self):
        self.update()
        return self.render()

    def update(self):
        pass

    def render(self):
        if self.template is None:
            template = getMultiAdapter((self, self.request), IPageTemplate)
            return template(self)
        return self.template()


class TemplateBasedPage(BaseTemplateBasedPage):
    """Template based page"""

    layout = getLayoutTemplate()

    def __call__(self):
        self.update()
        if self.layout is None:
            layout = getMultiAdapter((self, self.request), ILayoutTemplate)
            return layout(self)
        return self.layout()


class BaseIndexPage(TemplateBasedPage):
    """Base index page"""


class InnerPage(TemplateBasedPage):
    """Inner page"""

    implements(IInnerPage)


class ModalPage(TemplateBasedPage):
    """Modal page"""

    implements(IModalPage)


class ExceptionView(BrowserPage):
    """Base exception view"""

    @property
    def error_name(self):
        return self.context.__class__.__name__

    @property
    def error_message(self):
        return textToHTML(translate(getattr(self.context, 'message', u''), context=self.request) or
                          '\n'.join((str(arg) for arg in self.context.args)),
                          request=self.request)

    @property
    def error_datetime(self):
        return formatDatetime(tztime(datetime.utcnow()))

    @property
    def error_user(self):
        principal = self.request.principal
        return '%s (%s)' % (principal.title, principal.id)

    def __call__(self):
        self.request.response.setStatus(500)
        writer = getUtility(IJSONWriter)
        return writer.write({'status': 'messagebox',
                             'messagebox': {'status': 'error',
                                            'title': translate(_("An error occurred: %s"), context=self.request) %
                                                     self.error_name,
                                            'content': self.error_message,
                                            'number': self.error_datetime,
                                            'icon': 'fa fa-warning animated shake'}})


class UnauthorizedExceptionView(ExceptionView):
    """Unauthorized exception view"""

    def __call__(self):
        principal = self.request.principal
        auth = getUtility(IAuthentication)
        auth.unauthorized(principal.id, self.request)
        self.request.response.setStatus(200)
        parent = getParent(self.context, IPersistent)
        writer = getUtility(IJSONWriter)
        return writer.write({'status': 'modal',
                             'location': 'login-dialog.html?came_from=%s' % absoluteURL(parent, self.request)})



#
# Copyright (c) 2014 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.myams.interfaces import IMyAMSApplication
from ztfy.myams.interfaces.search import IMyAMSApplicationSearch
from ztfy.myams.viewlet.search.interfaces import ISearchViewlet

# import Zope3 packages
from zope.component import adapts
from zope.i18n import translate
from zope.interface import implements, Interface

# import local packages
from ztfy.baseskin.viewlet import ContentProviderBase
from ztfy.myams.layer import MyAMSLayer
from ztfy.utils.traversing import getParent


class SiteSearchViewlet(ContentProviderBase):
    """Site search content provider"""

    adapts(Interface, MyAMSLayer, Interface)
    implements(ISearchViewlet)

    search_config = None

    def update(self):
        app = getParent(self.context, IMyAMSApplication)
        if app is not None:
            self.search_config = IMyAMSApplicationSearch(app, None)

    @property
    def placeholder(self):
        if self.search_config is not None:
            return translate(self.search_config.site_search_placeholder, context=self.request)

    @property
    def handler(self):
        if self.search_config is not None:
            return self.search_config.site_search_handler


class MobileSearchViewlet(ContentProviderBase):
    """Mobile search content provider"""

    adapts(Interface, MyAMSLayer, Interface)
    implements(ISearchViewlet)

    search_config = None

    def update(self):
        app = getParent(self.context, IMyAMSApplication)
        if app is not None:
            self.search_config = IMyAMSApplicationSearch(app, None)

    @property
    def placeholder(self):
        if self.search_config is not None:
            return translate(self.search_config.mobile_search_placeholder, context=self.request)

    @property
    def handler(self):
        if self.search_config is not None:
            return self.search_config.mobile_search_handler



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from zope.interface import Interface, Attribute

# import local packages

from ztfy.myams import _


class ISearchViewlet(Interface):
    """Site search viewlet interface"""

    placeholder = Attribute(_("Search input placeholder"))

    handler = Attribute(_("Search handler"))



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces
from z3c.json.interfaces import IJSONWriter

# import local interfaces
from ztfy.baseskin.interfaces import IDialog
from ztfy.baseskin.interfaces.form import IGroupsBasedForm, IWidgetsGroup, IAJAXForm, IForm, IInnerSubForm, \
    IInnerTabForm, ICustomUpdateSubForm
from ztfy.myams.interfaces import IModalEditFormButtons, IModalAddFormButtons, IEditFormButtons, IAddFormButtons, \
    IModalDisplayFormButtons, IObjectData

# import Zope3 packages
from z3c.form import button
from z3c.formjs import ajax
from z3c.formui import form
from zope.component import getAdapters, getUtility
from zope.event import notify
from zope.i18n import translate
from zope.interface import implements
from zope.lifecycleevent import ObjectCreatedEvent, Attributes
from zope.schema.fieldproperty import FieldProperty
from zope.security.proxy import removeSecurityProxy

# import local packages
from ztfy.baseskin.form import FormObjectCreatedEvent, FormObjectModifiedEvent
from ztfy.utils.property import cached_property

from ztfy.myams import _


#
# Form widgets group
#

class WidgetsGroup(object):
    """Widgets group"""

    implements(IWidgetsGroup)

    def __init__(self, id, widgets=(), legend=None, help=None, css_class='', switch=False,
                 checkbox_switch=False, checkbox_field=None, hide_if_empty=False):
        assert (not checkbox_switch) or checkbox_field, "You must define checkbox field when using checkbox switch"
        self.id = id
        self.widgets = widgets
        self.legend = (legend is None) and id or legend
        self.help = help
        self._css_class = css_class
        self.switch = switch
        self.checkbox_switch = checkbox_switch
        self.checkbox_field = checkbox_field
        self.hide_if_empty = hide_if_empty

    @property
    def switchable(self):
        return self.switch or self.checkbox_switch

    @property
    def checkbox_widget(self):
        if self.checkbox_field is None:
            return None
        for widget in self.widgets:
            if widget.field is self.checkbox_field.field:
                return widget

    @cached_property
    def visible(self):
        if self.checkbox_switch:
            widget = self.checkbox_widget
            context = widget.context
            name = widget.field.getName()
            value = getattr(context, name, None)
            return bool(value)
        else:
            if not (self.switch and self.hide_if_empty):
                return True
            for widget in self.widgets:
                if not widget.ignoreContext:
                    field = widget.field
                    context = widget.context
                    name = field.getName()
                    value = getattr(context, name, None)
                    if value and (value != field.default):
                        return True
            return False

    @property
    def visible_widgets(self):
        for widget in self.widgets:
            if (self.checkbox_field is None) or (widget.field is not self.checkbox_field.field):
                yield widget

    @property
    def css_class(self):
        css_class = self._css_class
        if self.switch:
            if self.checkbox_switch:
                css_class += ' checker'
            else:
                css_class += ' switcher'
        return css_class

    @property
    def switcher_state(self):
        return 'on' if self.visible else 'off'

    @property
    def checker_state(self):
        return 'on' if self.visible else 'off'


def NamedWidgetsGroup(id, widgets, names=(), legend=None, help=None, css_class='', switch=False,
                      checkbox_switch=False, checkbox_field=None, hide_if_empty=False):
    """Create a widgets group based on widgets names"""
    return WidgetsGroup(id, [widgets.get(name) for name in names], legend, help, css_class, switch,
                        checkbox_switch, checkbox_field, hide_if_empty)


class GroupsBasedForm(object):
    """Groups based form"""

    implements(IGroupsBasedForm)

    def __init__(self):
        self._groups = []

    def addGroup(self, group):
        self._groups.append(group)

    @property
    def groups(self):
        result = self._groups[:]
        others = []
        for widget in self.widgets.values():
            found = False
            for group in result:
                if widget in group.widgets:
                    found = True
                    break
            if not found:
                others.append(widget)
        if others:
            result.insert(0, WidgetsGroup(None, others))
        return result


#
# AJAX form
#

class AjaxForm(ajax.AJAXRequestHandler):
    """Custom base class used to handle AJAX errors"""

    implements(IAJAXForm, IObjectData)

    object_data = None
    form_options = None

    def getFormOptions(self):
        if self.form_options:
            writer = getUtility(IJSONWriter)
            return writer.writer(self.form_options)

    def getAjaxErrors(self):
        errors = {'status': u'error',
                  'error_message': translate(self.status, context=self.request)}
        for error in self.errors:
            error.update()
            error = removeSecurityProxy(error)
            if hasattr(error, 'widget'):
                widget = removeSecurityProxy(error.widget)
                if widget is not None:
                    errors.setdefault('widgets', []).append({'name': widget.name,
                                                             'label': translate(widget.label, context=self.request),
                                                             'message': translate(error.message, context=self.request)})
                else:
                    errors.setdefault('messages', []).append({'message': translate(error.message, context=self.request)})
            else:
                errors.setdefault('messages', []).append(translate(error.message, context=self.request))
        return errors


#
# Base forms
#

def getFormWeight(form):
    try:
        return form.weight
    except AttributeError:
        return 0


class BaseForm(GroupsBasedForm, AjaxForm, form.Form):
    """Base add form"""

    implements(IForm)

    autocomplete = 'on'
    display_hints_on_widgets = True
    handle_upload = False

    subforms_legend = None

    css_class = 'ams-form form-horizontal'
    label_css_class = FieldProperty(IForm['label_css_class'])
    input_css_class = FieldProperty(IForm['input_css_class'])

    callbacks = {}

    def __init__(self, context, request):
        GroupsBasedForm.__init__(self)
        form.Form.__init__(self, context, request)

    def update(self):
        form.Form.update(self)
        self.getForms()
        [subform.update() for subform in self.subforms]
        [tabform.update() for tabform in self.tabforms]

    def isDialog(self):
        return IDialog.providedBy(self)

    def getForms(self, with_self=True):
        if not hasattr(self, 'subforms'):
            self.subforms = self.createSubForms()
        if not hasattr(self, 'tabforms'):
            self.tabforms = self.createTabForms()
        if with_self:
            return [self, ] + self.subforms + self.tabforms
        else:
            return self.subforms + self.tabforms

    def createSubForms(self):
        return sorted((adapter[1] for adapter in getAdapters((self, ), IInnerSubForm)), key=getFormWeight)

    def createTabForms(self):
        return sorted((adapter[1] for adapter in getAdapters((self, ), IInnerTabForm)), key=getFormWeight)

    def getWidgetCallback(self, widget):
        return self.callbacks.get(widget)

    def updateWidgets(self, prefix=None):
        form.Form.updateWidgets(self, prefix)
        self.getForms()
        [subform.updateWidgets(prefix) for subform in self.subforms]
        [tabform.updateWidgets(prefix) for tabform in self.tabforms]

    @property
    def errors(self):
        result = []
        for subform in self.getForms():
            result.extend(subform.widgets.errors)
        return result

    def updateContent(self, content, data):
        changes = form.applyChanges(self, content, data)
        self.getForms()
        for subform in self.subforms:
            if ICustomUpdateSubForm.providedBy(subform):
                updates = ICustomUpdateSubForm(subform).updateContent(content, data)
                if isinstance(updates, dict):
                    changes.update(updates)
            else:
                changes.update(form.applyChanges(subform, content, data))
        for tabform in self.tabforms:
            if ICustomUpdateSubForm.providedBy(tabform):
                updates = ICustomUpdateSubForm(tabform).updateContent(content, data)
                if isinstance(updates, dict):
                    changes.update(updates)
            else:
                changes.update(form.applyChanges(subform, content, data))
        return changes


#
# Add forms
#

class AddForm(BaseForm, form.AddForm):
    """Base add form class"""

    buttons = button.Buttons(IAddFormButtons)

    legend = _("Add form")
    handler = '/@@ajax/ajaxCreate'
    formErrorsMessage = _("There were some errors.")

    def updateActions(self):
        form.AddForm.updateActions(self)
        if 'add' in self.actions:
            self.actions['add'].addClass('btn-primary')

    @ajax.handler
    def ajaxCreate(self, action):
        writer = getUtility(IJSONWriter)
        self.updateWidgets()
        data, errors = self.extractData()
        if errors:
            return writer.write(self.getAjaxErrors())
        result = self.createAndAdd(data)
        return self.getSubmitOutput(writer, result)

    def createAndAdd(self, data):
        object = self.create(data)
        notify(ObjectCreatedEvent(object))
        self.add(object)
        self.updateContent(object, data)
        notify(FormObjectCreatedEvent(object, self))
        return object

    def getSubmitOutput(self, writer, changes):
        return writer.write({'status': 'reload',
                             'location': self.nextURL()})


class DialogAddForm(AddForm):
    """Base dialog add form"""

    implements(IDialog)

    buttons = button.Buttons(IModalAddFormButtons)
    dialog_class = 'modal-medium'


#
# Edit forms
#

class EditForm(BaseForm, form.EditForm):
    """Base edit form class"""

    buttons = button.Buttons(IEditFormButtons)

    legend = _("Edit form")
    handler = '/@@ajax/ajaxEdit'
    formErrorsMessage = _('There were some errors.')
    successMessage = _('Data successfully updated.')
    noChangesMessage = _('No changes were applied.')

    def updateActions(self):
        form.EditForm.updateActions(self)
        if 'submit' in self.actions:
            self.actions['submit'].addClass('btn-primary')

    @ajax.handler
    def ajaxEdit(self):
        return self.handleApply()

    def handleApply(self):
        writer = getUtility(IJSONWriter)
        self.updateWidgets()
        data, errors = self.extractData()
        if errors:
            return writer.write(self.getAjaxErrors())
        changes = self.applyChanges(data)
        return self.getSubmitOutput(writer, changes)

    def applyChanges(self, data):
        content = self.getContent()
        changes = self.updateContent(content, data)
        if changes:
            descriptions = []
            for interface, names in changes.items():
                descriptions.append(Attributes(interface, *names))
            notify(FormObjectModifiedEvent(content, self, *descriptions))
        return changes

    def getSubmitOutput(self, writer, changes):
        if changes:
            message = self.successMessage
        else:
            message = self.noChangesMessage
        return writer.write({'status': 'success',
                             'message': translate(message, context=self.request)})


class DialogEditForm(EditForm):
    """Base dialog edit form"""

    implements(IDialog)

    buttons = button.Buttons(IModalEditFormButtons)
    dialog_class = 'modal-medium'


#
# Display forms
#

class DisplayForm(BaseForm, form.DisplayForm):
    """Base display form class"""


class DialogDisplayForm(DisplayForm):
    """Base dialog display form"""

    implements(IDialog)

    buttons = button.Buttons(IModalDisplayFormButtons)
    dialog_class = 'modal-medium'



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.myams.viewlet.toplinks.interfaces import ITopLinksViewletManager, ITopLinksViewlet, ITopLinksMenu

# import Zope3 packages
from zope.i18n import translate
from zope.interface import implements

# import local packages
from ztfy.baseskin.viewlet import WeightViewletManagerBase, ViewletBase


class TopLinksViewletManager(WeightViewletManagerBase):
    """Top links viewlet manager"""

    implements(ITopLinksViewletManager)


class TopLinksViewlet(WeightViewletManagerBase, ViewletBase):
    """Top links viewlet"""

    implements(ITopLinksViewlet)

    label = u"Label:"
    css_class = u'top-menu'
    dropdown_label = u"Dropdown label"

    def __init__(self, context, request, view, manager=None):
        WeightViewletManagerBase.__init__(self, context, request, view)
        ViewletBase.__init__(self, context, request, view, manager)

    def update(self):
        WeightViewletManagerBase.update(self)


class TopLinksMenu(ViewletBase):
    """Top link menu"""

    implements(ITopLinksMenu)

    css_class = u""
    label = u"Menu label"
    click_handler = None
    url = u"#"
    data = {}

    def render(self):
        if self.css_class == u'divider':
            return u'<li class="divider"></li>'
        else:
            label = translate(self.label, context=self.request)
            return u'''<li class="%s">
                        <a href="%s" %s>%s</a>
                    </li>''' % (self.css_class,
                                self.url,
                                self.getDataAttributes(),
                                unicode(label, 'utf-8') if isinstance(label, str) else label)

    def getDataAttributes(self):
        data = self.data.copy()
        if self.click_handler:
            data.update({'data-ams-click-handler': self.click_handler})
        return ' '.join('%s=%s' % item for item in data.iteritems())



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.myams.viewlet.menu.interfaces import IMainMenusViewletManager, IMenu, IMenuItem

# import Zope3 packages
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty

# import local packages
from ztfy.baseskin.viewlet import WeightViewletManagerBase, ViewletBase


class MainMenusViewletManager(WeightViewletManagerBase):
    """Mains menus viewlet manager"""

    implements(IMainMenusViewletManager)


class Menu(WeightViewletManagerBase, ViewletBase):
    """Menu header"""

    implements(IMenu)

    header = FieldProperty(IMenu['header'])

    def __init__(self, context, request, view, manager=None):
        WeightViewletManagerBase.__init__(self, context, request, view)
        ViewletBase.__init__(self, context, request, view, manager)

    def update(self):
        WeightViewletManagerBase.update(self)

    def render(self):
        return ViewletBase.render(self)


class MenuItem(WeightViewletManagerBase, ViewletBase):
    """Menu viewlet"""

    implements(IMenuItem)

    css_class = FieldProperty(IMenuItem['css_class'])
    icon_class = FieldProperty(IMenuItem['icon_class'])
    label = FieldProperty(IMenuItem['label'])
    badge = FieldProperty(IMenuItem['badge'])
    badge_class = FieldProperty(IMenuItem['badge_class'])
    notice = FieldProperty(IMenuItem['notice'])
    notice_class = FieldProperty(IMenuItem['notice_class'])
    click_handler = FieldProperty(IMenuItem['click_handler'])
    url = FieldProperty(IMenuItem['url'])
    modal_target = FieldProperty(IMenuItem['modal_target'])
    data = FieldProperty(IMenuItem['data'])

    def __init__(self, context, request, view, manager=None):
        WeightViewletManagerBase.__init__(self, context, request, view)
        ViewletBase.__init__(self, context, request, view, manager)

    def update(self):
        WeightViewletManagerBase.update(self)

    def render(self):
        return ViewletBase.render(self)

    def getURL(self):
        return self.url

    def getDataAttributes(self):
        data = self.data.copy()
        if self.click_handler:
            data.update({'data-ams-click-handler': self.click_handler})
        return ' '.join('%s=%s' % item for item in data.iteritems())


class MenuDivider(ViewletBase):
    """Menu divider viewlet"""

    implements(IMenuItem)

    header = None
    css_class = 'divider'
    icon_class = None
    label = None
    badge = None
    badge_class = None
    notice = None
    notice_class = None
    click_handler = None
    url = None
    modal_target = False
    data = None

    def getURL(self):
        return None



#
# Copyright (c) 2014 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from z3c.table.table import Table

# import local packages


class BaseTable(Table):
    """Custom table"""

    data_attributes = {}

    @staticmethod
    def checkDataAttribute(attribute, source):
        if isinstance(attribute, (str, unicode)):
            return attribute
        elif callable(attribute):
            return attribute(source)
        else:
            return str(attribute)

    def getDataAttributes(self, element, source, column=None):
        attrs = self.data_attributes.get(element)
        if attrs:
            return ' '.join("%s='%s'" % (item[0], self.checkDataAttribute(item[1], source)) for item in attrs.iteritems())
        else:
            return ''

    def renderTable(self):
        return super(BaseTable, self).renderTable() \
                                     .replace('<table', '<table %s' % self.getDataAttributes('table', self))

    def renderRow(self, row, cssClass=None):
        return super(BaseTable, self).renderRow(row, cssClass) \
                                     .replace('<tr', '<tr %s' % self.getDataAttributes('tr', row[0][0]))

    def renderCell(self, item, column, colspan=0):
        return super(BaseTable, self).renderCell(item, column, colspan) \
                                     .replace('<td', '<td %s' % self.getDataAttributes('td', item, column))



#
# Copyright (c) 2012 Thierry Florac <tflorac AT onf.fr>
# All Rights Reserved.
#


# import standard packages
import pkg_resources
from persistent import Persistent

# import Zope3 interfaces

# import local interfaces
from zope.annotation.interfaces import IAnnotations
from zope.location.location import locate
from ztfy.myams.interfaces import IMyAMSApplication
from ztfy.myams.interfaces.configuration import IMyAMSStaticConfiguration, \
    IMyAMSConfigurationTarget, IMyAMSConfiguration

# import Zope3 packages
from zope.component import adapter, queryUtility
from zope.container.contained import Contained
from zope.interface import implementer, implements
from zope.schema.fieldproperty import FieldProperty
from ztfy.extfile.blob import BlobImage, BlobFile
from ztfy.file.property import FileProperty

# import local packages
from ztfy.utils.traversing import getParent


class MyAMSStaticConfiguration(object):
    """MyAMS static configuration"""

    implements(IMyAMSStaticConfiguration)

    application_package = 'ztfy.myams'
    application_name = u'MyAMS'
    include_top_links = True
    include_site_search = True
    include_mobile_search = True
    include_user_activity = True
    include_user_shortcuts = True
    include_logout_button = True
    include_minify_button = True
    include_flags = True
    include_menus = True
    include_ribbon = True
    include_reload_button = True
    body_css_class = u''

    @property
    def version(self):
        return pkg_resources.get_distribution(self.application_package).version


class MyAMSConfiguration(Persistent, Contained):
    """MyAMS user configuration"""

    implements(IMyAMSConfiguration)

    title = FieldProperty(IMyAMSConfiguration['title'])
    description = FieldProperty(IMyAMSConfiguration['description'])
    author = FieldProperty(IMyAMSConfiguration['author'])
    icon = FileProperty(IMyAMSConfiguration['icon'], klass=BlobImage, img_klass=BlobImage)
    logo = FileProperty(IMyAMSConfiguration['logo'], klass=BlobImage, img_klass=BlobImage)
    logo_title = FieldProperty(IMyAMSConfiguration['logo_title'])
    custom_css = FileProperty(IMyAMSConfiguration['custom_css'], klass=BlobFile)
    custom_js = FileProperty(IMyAMSConfiguration['custom_js'], klass=BlobFile)
    google_analytics_key = FieldProperty(IMyAMSConfiguration['google_analytics_key'])

    @property
    def static_configuration(self):
        application = getParent(self, IMyAMSApplication)
        if application is not None:
            return queryUtility(IMyAMSStaticConfiguration, name=application.configuration_name)


MYAMS_CONFIGURATION_KEY = 'ztfy.myams.configuration'


@adapter(IMyAMSConfigurationTarget)
@implementer(IMyAMSConfiguration)
def MyAMSConfigurationFactory(context):
    """MyAMS configuration factory"""
    annotations = IAnnotations(context)
    configuration = annotations.get(MYAMS_CONFIGURATION_KEY)
    if configuration is None:
        configuration = annotations[MYAMS_CONFIGURATION_KEY] = MyAMSConfiguration()
        locate(configuration, context, '++configuration++')
    return configuration
